require 'time.rb'

AT_ONLY_LEVEL_0 = 0
AT_ONLY_LEVEL_01 = 1
AT_ALL_LEVELS = 2
AT_ALSO_TASKS = 3

class KkWorksController < ApplicationController
    unloadable
    before_filter :authorize_global
    before_filter :ensure_permissions

    def initialize
        
        # Diary
        
        @spent_on_s = Date.today.to_s
        @time_from_s = ''
        @time_to_s = ''
        @issue_id_s = ''
        @activity_id = ''
        @comments = ''
        @error_text = []
        @editing_id = nil
        @history_depth = 1 # Week
        
        # Report 
        
        @alsoTasks = AT_ALL_LEVELS
        @user_kind = "U"
        @user_id = User.current.id
        
        @intervalMethod = "monthYear"
        
        @month = 1
        @year = 2011
        
        @dateFrom = nil
        @dateFrom_s = ""
        @dateTo = nil
        @dateTo_s = ""
    end

    def ensure_permissions()
        @can_view_works = !!User.current.allowed_to?(:kkworks_view_works, nil, {:global => true})
        @can_edit_works = !!User.current.allowed_to?(:kkworks_edit_works, nil, {:global => true})
        @can_view_self_reports = !!User.current.allowed_to?(:kkworks_view_self_report, nil, {:global => true})
        @can_view_others_reports = !!User.current.allowed_to?(:kkworks_view_others_report, nil, {:global => true})
        @can_view_reports = @can_view_self_reports or @can_view_others_reports
        if @can_view_others_reports
            @user_id = nil
        else
            @user_id = User.current.id
        end
        #logger.info("View Works: %s, Edit Works: %s, View Self Report: %s, View Others Report: %s" % [@can_view_works.to_s, @can_edit_works.to_s, @can_view_self_reports.to_s, @can_view_reports.to_s])
    end

    def extractParams(params)
        @comments = params[:comments]
        @issue_id_s = params[:issue_id].to_i
        @activity_id_s = params[:activity_id].to_i
        @spent_on_s = params[:spent_on]
        @time_from_s = params[:time_from]
        @time_to_s = params[:time_to]
    end

    def convertParams(alsoDate = true)
        # Assume success
        res = true

        # Convert date and times
        if alsoDate
            begin
                @spent_on = Date.parse(@spent_on_s)
            rescue
                @spent_on = nil
                @error_text << l(:error_invalid_spent_on)
                res = false
            end
        end
        begin
            @time_from = Time.parse(@time_from_s)
        rescue
            @time_from = nil
            @error_text << l(:error_invalid_time_from)
            res = false
        end
        begin
            @time_to = Time.parse(@time_to_s)
        rescue
            @time_to = nil
            @error_text << l(:error_invalid_time_to)
            res = false
        end

        # Calculate a spent time
        if @time_from != nil and @time_to != nil
            if @time_to.hour == 0 and @time_to.min == 0
                @spent_time = 24.0 + (@time_to - @time_from) / 60 / 60
            elsif @time_from < @time_to
                @spent_time = (@time_to - @time_from) / 60 / 60
            else
                @error_text << l(:error_end_time_less_than_start)
                res = false
            end
        end

        # Check issue ID
        if @issue_id_s == nil or @issue_id_s <= 0
            @error_text << l(:error_no_issue_selected)
            res = false
        end

        # Check activity ID
        if @activity_id_s == nil or @activity_id_s <= 0
            @error_text << l(:error_no_activity_selected)
            res = false
        end

        # Avoid duplicate entries
        if res
            entries = TimeEntry.find(
                :first,
                :conditions => ["#{TimeEntry.table_name}.user_id = ? and #{TimeEntry.table_name}.spent_on = ? and #{TimeEntry.table_name}.time_from < ? and #{TimeEntry.table_name}.time_to > ?", User.current.id, @spent_on, @time_to, @time_from]
            )
            if entries != nil
                @error_text << l(:error_overlaping_timespans)
                res = false
            end
        end

        # Result
        if !res
            @error_text.each do |e|
                logger.info("Save entry: Error: " + e)
            end
        end

        res
    end

    def index
        # Ensure the history depth from the session
        if session.has_key?(:kk_works_history_depth)
            @history_depth = session[:kk_works_history_depth]
        end
        # Ensure the selected user ID from the session
        if session.has_key?(:kk_works_diary_user_id) && @can_view_others_reports
            @user_id = session[:kk_works_diary_user_id]
        else
            session[:kk_works_diary_user_id] = @user_id = User.current.id
        end
    end

    def delete
        # Ensure the history depth from the session
        if session.has_key?(:kk_works_history_depth)
            @history_depth = session[:kk_works_history_depth]
        end
        # Ensure the selected user ID from the session
        session[:kk_works_diary_user_id] = @user_id = User.current.id

        # Locate and delete the specified record
        dr = TimeEntry.find(params[:id])
        dr.delete

        # Redirect back to the index
        flash.now[:notice] = l :ok_effort_log_deleted
        render :action => 'index'
    end

    def edit
        # Ensure the history depth from the session
        if session.has_key?(:kk_works_history_depth)
            @history_depth = session[:kk_works_history_depth]
        end
        # Ensure the selected user ID from the session
        session[:kk_works_diary_user_id] = @user_id = User.current.id

        @editing_id = params[:id].to_i

        # Redirect back to the index
        flash.now[:notice] = l :ok_effort_log_deleted
        render :action => 'index'
    end

    def save_edit
        # Ensure the history depth from the session
        if session.has_key?(:kk_works_history_depth)
            @history_depth = session[:kk_works_history_depth]
        end
        # Ensure the selected user ID from the session
        session[:kk_works_diary_user_id] = @user_id = User.current.id

        # Extract and process parameters
        logger.info("Save entry: Extracting params")
        extractParams(params)

        logger.info("Save entry: Converting params")
        if convertParams(false)
            id = params[:id].to_s
            # Locate and delete the specified record
            nr = TimeEntry.find(id)
            logger.info("Save entry: Updating work record: ID=%d->%d, Issue=%d->%d" % [nr.id, id, nr.issue_id, @issue_id_s])
            nr.time_from = @time_from
            nr.time_to = @time_to
            nr.project_id = nil
            nr.issue_id = @issue_id_s
            nr.hours = @spent_time
            nr.comments = @comments
            nr.activity_id = @activity_id_s
            logger.info("Save entry: Saving work record")
            nr.save
            flash.now[:notice] = l :ok_effort_log_created
        end

        @spent_on_s = Date.today.to_s

        # Redirect back to the index
        render :action => 'index'
    end

    def entry
        # Ensure the history depth from the session
        if session.has_key?(:kk_works_history_depth)
            @history_depth = session[:kk_works_history_depth]
        end
        # Ensure the selected user ID from the session
        session[:kk_works_diary_user_id] = @user_id = User.current.id

        # Extract and process parameters
        extractParams(params)

        if convertParams
            # Insert new record
            nr = TimeEntry.new
            nr.time_from = @time_from
            nr.time_to = @time_to
            nr.user_id = User.current.id
            nr.issue_id = @issue_id_s
            nr.hours = @spent_time
            nr.comments = @comments
            nr.activity_id = @activity_id_s
            nr.spent_on = @spent_on
            nr.save
            flash.now[:notice] = l :ok_effort_log_created
        end

        # Redirect back to the index
        render :action => 'index'
    end

    def history_depth
        session[:kk_works_history_depth] = @history_depth = params[:history_depth].to_i
        session[:kk_works_diary_user_id] = @user_id = params[:user_id].to_i

        # Redirect back to the index
        render :action => 'index'
    end

    def report
        
        # User
        if params.has_key?(:user_id)
            @user_id = params[:user_id]
        elsif session.has_key?(:kk_works_user_id)
            @user_id = session[:kk_works_user_id]
        else
            @user_id = "U" + User.current.id.to_s
        end
        @user_kind = @user_id[0,1]
        @user_id = @user_id[1,9999]
        if @user_kind == "U"
            @user_id = @user_id.to_i
            session[:kk_works_user_id] = "U" + @user_id.to_s
        elsif @user_kind == "G"
            @user_id = @user_id.to_i
            session[:kk_works_user_id] = "G" + @user_id.to_s
        elsif @user_kind == "A"
            @user_id = nil
            session[:kk_works_user_id] = "A"
        else
            @user_id = User.current.id
            @user_kind = "U"
            session[:kk_works_user_id] = "U" + @user_id.to_s
        end
        @user_id_s = @user_kind + @user_id.to_s
        
        # Year
        if params.has_key?(:year)
            @year = params[:year].to_i
        elsif session.has_key?(:kk_works_year)
            @year = session[:kk_works_year]
        else
            @year = Date.today.year
        end
        session[:kk_works_year] = @year
        
        # Month
        if params.has_key?(:month)
            @month = params[:month].to_i
        elsif session.has_key?(:kk_works_month)
            @month = session[:kk_works_month]
        else
            @month = Date.today.month
        end
        session[:kk_works_month] = @month
        
        # Also tasks
        if params.has_key?(:alsoTasks)
            @alsoTasks = params[:alsoTasks].to_i
        elsif session.has_key?(:kk_works_alsoTasks)
            @alsoTasks = session[:kk_works_alsoTasks]
        else
            @alsoTasks = AT_ALL_LEVELS
        end
        session[:kk_works_alsoTasks] = @alsoTasks
        
        # Date from
        if params.has_key?(:dateFrom)
            @dateFrom_s = params[:dateFrom]
            begin
                @dateFrom = Date.parse(@dateFrom_s)
            rescue
                @dateFrom = nil
                @dateFrom_s = ""
            end            
        elsif session.has_key?(:kk_works_dateFrom)
            @dateFrom_s = session[:kk_works_dateFrom]
            begin
                @dateFrom = Date.parse(@dateFrom_s)
            rescue
                @dateFrom = nil
                @dateFrom_s = ""
            end                        
        else
            @dateFrom = nil
            @dateFrom_s = ""
        end
        session[:kk_works_dateFrom] = @dateFrom_s
                    
        # Date to
        if params.has_key?(:dateTo)
            @dateTo_s = params[:dateTo]
            begin
                @dateTo = Date.parse(@dateTo_s)
            rescue
                @dateTo = nil
                @dateTo_s = ""
            end            
        elsif session.has_key?(:kk_works_dateTo)
            @dateTo_s = session[:kk_works_dateTo]
            begin
                @dateTo = Date.parse(@dateTo_s)
            rescue
                @dateTo = nil
                @dateTo_s = ""
            end                        
        else
            @dateTo = nil
            @dateTo_s = ""
        end
        session[:kk_works_dateTo] = @dateTo_s
                    
        # Interval method
        if params.has_key?(:intervalMethod)
            @intervalMethod = params[:intervalMethod]
        elsif session.has_key?(:kk_works_intervalMethod)
            @intervalMethod = session[:kk_works_intervalMethod]
        else
            @intervalMethod = "monthYear"
        end
        if @intervalMethod != "monthYear" and @intervalMethod != "fromTo"
            @intervalMethod = "monthYear"
        end            
        if @intervalMethod == "monthYear"
            @dateTo = nil
            @dateFrom = nil
        elsif @dateFrom == nil or @dateTo == nil
            @intervalMethod = "monthYear"
            @dateTo = nil
            @dateFrom = nil
        end
        session[:kk_works_intervalMethod] = @intervalMethod
    end
end
